// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Olivier Pierard, Gaetan Compere, Jean-Francois Remacle
// -------------------------------------------------------------------

#ifndef _H_EDGESPLITOP
#define _H_EDGESPLITOP

#include "MAdOperatorBase.h"

namespace MAd {

  // -------------------------------------------------------------------
  class edgeSplitOp: public MAdOperatorBase
  {
  public:

    edgeSplitOp(pMesh, DiscreteSF *);
    edgeSplitOp(const edgeSplitOp &);
    ~edgeSplitOp() {}

    operationType type() const { return MAd_ESPLIT; }

    void setSplitEdge(pEdge);
    void setSplitPos(const double[3]);

    void getCavity(pPList *) const;

    void apply();

  private:

    bool checkConstraints() const;
    bool checkGeometry();
    bool evaluateShapes();
    bool evaluateShapes2D();
    void evaluateLengths() const;

  private:

    pEdge edge;
    double xyz[3];
  };

  // -------------------------------------------------------------------
  inline edgeSplitOp::edgeSplitOp(const edgeSplitOp & _es):
    MAdOperatorBase(_es)
  {
    edge = _es.edge;
    xyz[0] = _es.xyz[0];
    xyz[1] = _es.xyz[1];
    xyz[2] = _es.xyz[2];
  }

  // -------------------------------------------------------------------
  inline edgeSplitOp::edgeSplitOp(pMesh _m, DiscreteSF * _sf):
    MAdOperatorBase(_m,_sf), edge(NULL)
  {
    xyz[0] = 0.;
    xyz[1] = 0.;
    xyz[2] = 0.;
  }

  // -------------------------------------------------------------------
  inline void edgeSplitOp::setSplitEdge(pEdge _edge)
  {
    edge = _edge;
  }

  // -------------------------------------------------------------------
  inline void edgeSplitOp::setSplitPos(const double _xyz[3]) 
  {
    xyz[0] = _xyz[0];
    xyz[1] = _xyz[1];
    xyz[2] = _xyz[2];
  } 

  // -------------------------------------------------------------------

}

#endif
