// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Jean-Francois Remacle, Gaetan Compere
// -------------------------------------------------------------------

#ifdef _HAVE_GMSH_

#include "GmshModel.h"
#include "MAdMessage.h"

#include "gmsh/discreteRegion.h"
#include "gmsh/discreteFace.h"
#include "gmsh/discreteEdge.h"
#include "gmsh/discreteVertex.h"

namespace MAd {

  // -------------------------------------------------------------------
  MAdGEntity * GmshModel::getEntityByTag(int dim, int tag)
  {
    switch(dim) {
    case 3: return getRegionByTag(tag);
    case 2: return getFaceByTag(tag);
    case 1: return getEdgeByTag(tag);
    case 0: return getVertexByTag(tag);
    }
  }

  // -------------------------------------------------------------------
  MAdGRegion * GmshModel::getRegionByTag(int tag)
  {
    GRegion * pGR = model->getRegionByTag(tag);
    if ( !pGR ) {
      MAdMsgSgl::instance().warning(__LINE__,__FILE__,"Region not found");
    }
    return pGR;
  }

  // -------------------------------------------------------------------
  MAdGFace * GmshModel::getFaceByTag(int tag)
  {
    GFace * pGF = model->getFaceByTag(tag);
    if ( !pGF ) {
      MAdMsgSgl::instance().warning(__LINE__,__FILE__,"Surface not found");
    }
    return pGF;
  }

  // -------------------------------------------------------------------
  MAdGEdge * GmshModel::getEdgeByTag(int tag)
  {
    GEdge * pGE = model->getEdgeByTag(tag);
    if ( !pGE ) {
      MAdMsgSgl::instance().warning(__LINE__,__FILE__,"Line not found");
    }
    return pGE;
  }

  // -------------------------------------------------------------------
  MAdGVertex * GmshModel::getVertexByTag(int tag)
  {
    GVertex * pGV = model->getVertexByTag(tag);
    if ( !pGV ) {
      MAdMsgSgl::instance().warning(__LINE__,__FILE__,"Point not found");
    }
    return pGV;
  }

  // -------------------------------------------------------------------
}

#endif
