// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Jean-Francois Remacle, Gaetan Compere
// -------------------------------------------------------------------

#ifndef _H_GMSHMODEL
#define _H_GMSHMODEL

#ifdef _HAVE_GMSH_

#include "MAdModel.h"
#include "GmshEntities.h"

#include "gmsh/GModel.h"
#include <list>

namespace MAd {

  // -------------------------------------------------------------------
  class GmshModel : public MAdModel {

  public:

    GmshModel(std::string name=""): 
      MAdModel(name)
    { 
      model = new GModel(_name);
      GModel::current(GModel::list.size() - 1);
    }
    ~GmshModel() { if (model) delete model; }

    ModelType type() const { return GMSHMODEL; }

    // Gmsh mesh file format
    int readMSH(const std::string &filename) { return model->readMSH(filename); }
  
    // Gmsh native CAD format
    int readGEO(const std::string &filename) { return model->readGEO(filename); }
    int writeGEO(const std::string &filename) { return model->writeGEO(filename); }
  
    // OCC formats
    int readSTEP(const std::string &filename) { return model->readOCCSTEP(filename); }
    int readBREP(const std::string &filename) { return model->readOCCBREP(filename); }
    int readIGES(const std::string &filename) { return model->readOCCIGES(filename); }

    GModel * getModel() { return model; }

    // get the number of entities in this model
    int getNumRegions()  const { return model->getNumRegions();  }
    int getNumFaces()    const { return model->getNumFaces();    }
    int getNumEdges()    const { return model->getNumEdges();    }
    int getNumVertices() const { return model->getNumVertices(); }

    // find the entity with the given tag and create it if not found
    MAdGEntity * getEntityByTag(int dim, int tag);
    MAdGRegion * getRegionByTag(int);
    MAdGFace   * getFaceByTag  (int);
    MAdGEdge   * getEdgeByTag  (int);
    MAdGVertex * getVertexByTag(int);

    // get an iterator initialized to the first/last entity in this model.
    riter firstRegion() { return model->firstRegion(); }
    fiter firstFace()   { return model->firstFace(); }
    eiter firstEdge()   { return model->firstEdge(); }
    viter firstVertex() { return model->firstVertex(); }
    riter lastRegion()  { return model->lastRegion(); }
    fiter lastFace()    { return model->lastFace(); }
    eiter lastEdge()    { return model->lastEdge(); }
    viter lastVertex()  { return model->lastVertex(); }

  private:
    GModel * model;

  };

}

// -------------------------------------------------------------------

#endif

#endif
